﻿//////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011 Dynastream Innovations Inc.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
//////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Timers;
using System.Threading;

namespace AutoSharedDemoApplication
{
    //Delegates for thread-safe calls
    delegate void dDisplay(Control control, String msg);
    delegate void dPicutre(Boolean torf);

    public partial class SlaveForm : Form
    {
        #region Constants
        /// <summary>
        /// Data Source Types
        /// </summary>
        private enum SourceTypes
        {
            UNDEFINED = 0x00,
            DIGITAL = 0x40,
            A_TO_D, 
            COUNTER, 
            RELAY, 
        }
        private enum SinkTypes
        {
            DIGITAL_OUT = 0x01, 
            RELAY_MASK,
        }
        private enum AtoDSubTypes
        {
            TEMPERATURE = 0xFE,
            BATT_VOLTAGE, 
        }
        #endregion

        System.Timers.Timer timer1 = new System.Timers.Timer();

        byte[] aucDataToSend = new byte[8]; //Holds the slave's shared address as well as the data to send to the slave
        ushort usDeviceType = (ushort)SourceTypes.UNDEFINED; //Set default device type to UNDEFINED 
        bool bUpdateLED = false; 

        /// <summary>
        /// Constructor
        /// </summary>
        public SlaveForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Allow only valid Hex numbers to be entered as data on form  
        /// </summary>
        /// <param name="sender">Object raising the event</param>
        /// <param name="e">Key event arguments</param>
        private void scriptTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode > Keys.F && e.KeyCode <= Keys.Z && !(e.Control) && !(e.Alt))
                e.SuppressKeyPress = true; //Suppress any Keys that aren't 0-9 or A-F
        }

        /// <summary>
        /// Handles when the Send button is clicked
        /// </summary>
        /// <param name="sender">Object raising event</param>
        /// <param name="e">Event arguments</param>
        private void sendButton_Click(object sender, EventArgs e)
        {
            string s = sharedAddress.Text + scriptTextBox.Text;
            string t = "";
            bool newbyte = false;
            byte temp = 0;
            int temp2 = 0;
            byte value = 0;


            if (!scriptTextBox.MaskFull)
            {
                MessageBox.Show("Data to send incorrect. Please enter 6 Hexadecimal bytes and try again", "Invalid Data", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }

            //Conver the string to send to an array of bytes, with the first two bytes being the slave's shared address
            for (int i = 0, k = 0; i < 32; i++)
            {
                temp = 0;
                t = s.Substring(i, 1);
                while (!(t.Equals("[") || t.Equals("]")))
                {
                    value = HexValue(t);
                    if (!newbyte)
                    {
                        temp2 = (int)value << 4;
                        newbyte = true;
                    }
                    else
                    {
                        temp = value;
                        temp += (byte)temp2;
                        newbyte = false;
                        aucDataToSend[k] = temp;
                        k++;
                    }
                    if (i == 0)
                        break;
                    i++;
                    t = s.Substring(i, 1);
                }
            }

            Display(sendButton, "Pending");
        }

        /// <summary>
        /// Sets the value of a given string to its hexadecimal value
        /// </summary>
        /// <param name="t">character (0 through 9 or A through F)</param>
        /// <returns>returns the decimal value of the character</returns>
        private byte HexValue(string t)
        {
            byte value;
            if (t.Contains("A"))
                value = 10;
            else
                if (t.Contains("B"))
                    value = 11;
                else
                    if (t.Contains("C"))
                        value = 12;
                    else
                        if (t.Contains("D"))
                            value = 13;
                        else
                            if (t.Contains("E"))
                                value = 14;
                            else
                                if (t.Contains("F"))
                                    value = 15;
                                else
                                    value = byte.Parse(t);

            return value;

        }

        /// <summary>
        /// Set the type of the device (i.e. Temperature Sensor)
        /// </summary>
        /// <param name="ucDeviceType">byte representing the type data type of device</param>
        /// <param name="ucSubType">byte representing the data being sent</param>
        private void SetDeviceType(byte ucDeviceType, byte ucSubType)
        {
            if (usDeviceType == ((ucDeviceType << 8) | ucSubType))
                return;
            usDeviceType = (ushort)((ucDeviceType << 8) | ucSubType);

            switch (ucDeviceType)
            {
                case (byte)SourceTypes.A_TO_D://Analog data
                    {
                        switch (ucSubType)
                        {
                            case 0:
                            case (byte)AtoDSubTypes.TEMPERATURE: //Temperature sensor
                                {
                                    Display(SensorType, "Temperature");
                                    Display(typeTextBox, "Temperature: ");
                                    break;
                                }
                            case (byte)AtoDSubTypes.BATT_VOLTAGE: //Battery Voltage
                                {
                                    Display(SensorType, "Battery Voltage");
                                    break;
                                }
                            default:
                                {
                                    Display(SensorType,  "Analog");
                                    Display(typeTextBox, "Value :");
                                    break;
                                }
                        }
                        break;
                    }

                case (byte)SourceTypes.COUNTER: 
                    {
                        Display(SensorType, "Counter");
                        Display(typeTextBox, "Count: ");
                        break;
                    }
                case (byte)SourceTypes.DIGITAL: 
                    {
                        Display(SensorType, "Digital");
                        break;
                    }
                case (byte)SourceTypes.UNDEFINED:
                default:
                    {
                        Display(SensorType, "Undefined");
                        break;
                    }
            }
        }

        /// <summary>
        /// Thread-safe call to change control text
        /// </summary>
        /// <param name="control">Control to change</param>
        /// <param name="msg">Text to display</param>
        private void Display(Control control, String msg)
        {
            if (control.InvokeRequired)
            {
                dDisplay d = new dDisplay(Display);
                Invoke(d, new object[] { control, msg });
            }
            else
            {
                control.Text = msg;
            }
        }

        /// <summary>
        /// Decode and Display the data received from the slave
        /// </summary>
        /// <param name="aucData"></param>
        internal void DisplayData(byte[] aucData)
        {
            ushort usTemp;
            string strTemp = "[" + aucData[0].ToString("X2") + "][" + aucData[1].ToString("X2") + "][" + aucData[2].ToString("X2") + "][" + aucData[3].ToString("X2") + "]["
                                 + aucData[4].ToString("X2") + "][" + aucData[5].ToString("X2") + "][" + aucData[6].ToString("X2") + "][" + aucData[7].ToString("X2") + "]";
            //Display the data
            Display(Data, strTemp);

            SetDeviceType(aucData[2], aucData[3]);

            switch (aucData[2])
            {
                case (byte)SourceTypes.DIGITAL:
                    {
                        //Not supported in Demo
                        break;
                    }
                case (byte)SourceTypes.A_TO_D:
                    {
                        switch (aucData[3])
                        {
                            case (byte)AtoDSubTypes.TEMPERATURE:
                                {
                                    usTemp = (aucData[6]);
                                    usTemp += (ushort)(aucData[7] << 8);
                                    Display(dataTextBox, (usTemp / 100.0).ToString() + ((char)176).ToString() + "C");
                                    break;
                                }
                            case (byte)AtoDSubTypes.BATT_VOLTAGE:
                                {
                                    //Not supported in Demo
                                    break;
                                }
                            default:
                                {
                                    usTemp = (ushort)((int)aucData[6] << 8);
                                    usTemp += (ushort)(aucData[7]);
                                    Display(dataTextBox, usTemp.ToString());
                                    break;
                                }
                        }
                        break;
                    }
                case (byte)SourceTypes.COUNTER:
                    {
                        usTemp = (ushort)((int)aucData[6] << 8);
                        usTemp += (ushort)(aucData[7]);
                        Display(dataTextBox, usTemp.ToString());
                        break;
                    }
                default:
                    {
                        break;
                    }
            }

            //Eveytime data is received from the slave, indicate by making picture visible for 50ms
            Picture(true);
            System.Threading.Thread.Sleep(50);
            Picture(false);

            
        }

        /// <summary>
        /// Handles when the checkbox is clicked
        /// </summary>
        /// <param name="sender">Object raising event</param>
        /// <param name="e">Event arguments</param>
        private void slaveLEDCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            //Tell the slave to update LED
            bUpdateLED = true;
        }

        /// <summary>
        /// Gets the Data to be sent to the slave
        /// </summary>
        /// <param name="aucData">Data to be sent on the channel</param>
        /// <param name="bGetChannelData">True if the master needs to send data to the slave, false otherwise</param>
        /// <returns>true when send button has been clicked, false otherwise</returns>
        internal bool GetChannelData(ref byte[] aucData, ref bool bGetChannelData)
        {
            if (sendButton.Text.Equals("Pending"))
            {
                //collect that data to send from the frame
                Array.Copy(aucDataToSend, 2, aucData, 0, 6);
                bGetChannelData = true;
                return true;
            }
            else if (bUpdateLED)
            {
                aucData[0] = (byte)SourceTypes.DIGITAL; 
                for(int i = 1; i < 4; i++)
                    aucData[i] = 0x00; //Set next three bytes to 0x00
                aucData[4] = 0x01; 
                aucData[5] = slaveLEDCheckBox.Checked ? (byte)0x00 : (byte)0x01; 
                //Update the LED
                bUpdateLED = false;
                bGetChannelData = true;
            }

            return false;
        }

        /// <summary>
        /// Thread-safe call to update the checkmark picture
        /// </summary>
        /// <param name="torf">true if the picture is to be visible, false otherwise</param>
        private void Picture(Boolean torf)
        {
            if (pictureBox1.InvokeRequired)
            {
                dPicutre d = new dPicutre(Picture);
                Invoke(d, new object[] {torf});
            }
            else
            {
                pictureBox1.Visible = torf;
            }
        }

 
    }

}
